package com.psmreborn.pkga;

import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Handler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;

public class NoPayStationParser {
    private final File psmGamesFile;
    private final boolean forceRefresh;
    public NoPayStationParser(File gamesFile, boolean refresh){
        this.psmGamesFile = gamesFile;
        this.forceRefresh = refresh;
    }

    public interface DoneCallback {
        void call();
    }

    public void startParsing(Activity activity, DoneCallback doneCallback) {
        Handler handler = new Handler(activity.getMainLooper());

        ProgressDialog dialog = Dialogs.getProgressDialog(activity);
        dialog.setTitle("Refreshing Items...");
        dialog.setMessage("Downloading index ...");
        dialog.setIndeterminate(true);
        dialog.setCancelable(false);
        dialog.show();

        new Thread(() -> {
            try {
                download();
                final ArrayList<Game> games = parse();
                handler.post(() -> {
                    dialog.dismiss();
                    ((MainActivity) activity).updateGames(games);
                });
            } catch (final Exception e) {
                handler.post(() -> {
                    dialog.dismiss();
                    showErrorDialog(activity, e.toString());
                });
            }
            handler.post(() -> {
                doneCallback.call();
            });
        }).start();
    }

    private static void copyTo(InputStream inpStream, OutputStream outStream) throws IOException {
        byte[] buffer = new byte[8192];
        int totalRead;

        while(true){
            totalRead = inpStream.read(buffer, 0, buffer.length);
            if(totalRead == -1)
                break;
            outStream.write(buffer, 0, totalRead);
        }

        outStream.flush();
    }
    private void download() throws IOException {
        if(!forceRefresh && psmGamesFile.exists()){
            return;
        }

        URL url = new URL("http://nopaystation.com/tsv/PSM_GAMES.tsv");
        URLConnection connection = url.openConnection();
        connection.connect();

        InputStream input = new BufferedInputStream(url.openStream(), 8192);
        //noinspection IOStreamConstructor
        OutputStream output = new FileOutputStream(this.psmGamesFile);

        copyTo(input, output);
        output.close();
        input.close();
    }

    private ArrayList<Game> parse() throws IOException {
        BufferedReader tsvReader = new BufferedReader(new FileReader(this.psmGamesFile));
        tsvReader.readLine(); // skip the first line

        ArrayList<Game> games = new ArrayList<>();

        String line = tsvReader.readLine();
        while(line != null) {
            String[] values = line.split("\t");

            if(values.length >= 4){
                String titleId = values[0];
                String title = values[2];
                String downloadUrl = values[3];
                String zRif = values[4];

                if(!title.contains("[Unity]") && !zRif.equals("MISSING")){
                    games.add(new Game(titleId, title, downloadUrl, zRif));
                }
            }

            line = tsvReader.readLine();
        }

        return games;
    }

    private void showErrorDialog(Activity activity, String errorMessage) {
        Dialogs.getAlertBuilder(activity)
                .setTitle("Error Obtaining PSM_GAMES.TSV..")
                .setMessage(errorMessage)
                .setCancelable(false)
                .setPositiveButton("OK", null)
                .show();
    }
}
