package com.psmreborn.pkga;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Environment;
import android.os.Handler;
import android.os.PowerManager;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.zip.DataFormatException;

import yuv.pink.pkgextract.Pkg;

public class PsmGameDownloader {
    private final File psmApplicationsFolder;
    private final Game gameDownloading;
    private final File outputDir;
    private final String psmSdcardFolder = new File(new File(Environment.getExternalStorageDirectory(), "psm"), "download").getAbsolutePath();
    private final String psmGamesFolder = new File(new File(new File(new File(new File(Environment.getExternalStorageDirectory(), "Android"), "data"), "com.playstation.psstore"), "files"), "psm").getAbsolutePath();

    public PsmGameDownloader(Game downloadGame){
        this.gameDownloading = downloadGame;

        // get psm data folder ...
        if(Build.VERSION.SDK_INT >= 30)  {
            this.psmApplicationsFolder = new File(psmSdcardFolder);
        } else {
            this.psmApplicationsFolder = new File(psmGamesFolder);
        }
        //noinspection ResultOfMethodCallIgnored
        this.psmApplicationsFolder.mkdirs();

        this.outputDir = new File(this.psmApplicationsFolder, this.gameDownloading.titleId);
    }



    public boolean exists()
    {

        if(Build.VERSION.SDK_INT >= 30 && RootUtil.fileExistRoot(new File(psmGamesFolder, this.gameDownloading.titleId).getAbsolutePath()))
            return true;

        if(outputDir.exists())
            return true;

        return false;
    }

    public void startDownload(Activity activity) {
        Handler handler = new Handler(activity.getMainLooper());

        PowerManager pm = (PowerManager) activity.getApplicationContext().getSystemService(Context.POWER_SERVICE);
        PowerManager.WakeLock wakeLock;

        if(pm != null){
            wakeLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK, "pkga:downloading");
            if (!wakeLock.isHeld()) wakeLock.acquire();
        } else {
            wakeLock = null;
        }

        ProgressDialog dialog = Dialogs.getProgressDialog(activity);
        dialog.setTitle("Downloading: "+gameDownloading.getPkgFilename());
        dialog.setMessage(gameDownloading.title);
        dialog.setIndeterminate(false);
        dialog.setMax(100);
        dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        dialog.setCancelable(true);
        dialog.show();


        AtomicBoolean cancelled = new AtomicBoolean(false);
        dialog.setOnCancelListener(dialogInterface -> {
            cancelled.set(true);
        });

        new Thread(() -> {
            try {
                extractRif();
                download(dialog, handler, cancelled);

                // use root to write to /sdcard/android/data/ ...
                if(Build.VERSION.SDK_INT >= 30)
                    RootUtil.moveWithRoot(outputDir.getAbsolutePath(), psmGamesFolder);
                handler.post(() -> {
                    dialog.dismiss();
                    if(wakeLock != null) {
                        wakeLock.release();
                    }
                    if(cancelled.get()) {
                        showDoneDialog(activity, "Cancelled");
                    } else {
                        showDoneDialog(activity, "complete");
                    }
                });
            } catch (final Exception e) {
                handler.post(() -> {
                    dialog.dismiss();
                    showErrorDialog(activity, "Downloading", e.toString());
                });
            }
        }).start();
    }

    private void download(ProgressDialog dialog, Handler handler, AtomicBoolean cancelled) throws Exception {
        URL url = new URL(gameDownloading.downloadUrl);
        URLConnection connection = url.openConnection();
        connection.connect();
        int fileLength = connection.getContentLength();
        if(cancelled.get()) {
            return;
        }

        //noinspection ResultOfMethodCallIgnored
        outputDir.mkdirs();

        final int[] prevProgress = {0};
        InputStream input = new ProgressInputStream(new BufferedInputStream(url.openStream(), 8192),
                bytesRead -> {
                    int progress = (int) ((bytesRead * 100) / fileLength);
                        if(prevProgress[0] != progress) {
                            handler.post(() -> dialog.setProgress(progress));
                            prevProgress[0] = progress;
                        }
                    }
        );
        Pkg.Read(input, (file, reader) -> {
            if(cancelled.get()) {
                return false;
            }

            if(file.name.equals("contents")) return true;
            File outputPath = new File(outputDir, file.name.substring(file.name.indexOf("/")+1));
            if(file.flags == 4) {
                //noinspection ResultOfMethodCallIgnored
                outputPath.mkdirs();
                return true;
            }
            //noinspection IOStreamConstructor
            OutputStream output = new FileOutputStream(outputPath);
            copyData(reader, output, file.size);
            output.flush();
            output.close();
            return true;
        });
        input.close();
    }

    private void extractRif() throws IOException, DataFormatException {
        byte[] fakeRif = gameDownloading.getFakeRif();

        File outputLicenseFolder = new File(new File(this.psmApplicationsFolder, this.gameDownloading.titleId), "License");
        File outputLicenseFile = new File(outputLicenseFolder, "FAKE.rif");
        //noinspection ResultOfMethodCallIgnored
        outputLicenseFolder.mkdirs();

        FileOutputStream output = new FileOutputStream(outputLicenseFile);
        output.write(fakeRif, 0, fakeRif.length);
        output.close();
    }

    public static void copyData(InputStream inputStream, OutputStream outputStream, long sizeToCopy) throws IOException {
        byte[] buffer = new byte[512];
        long totalBytesCopied = 0;

        while(totalBytesCopied < sizeToCopy) {
            int chunkLength = (int) Math.min(buffer.length, sizeToCopy-totalBytesCopied);
            int chunkRead = inputStream.read(buffer, 0, chunkLength);
            outputStream.write(buffer, 0, chunkRead);
            totalBytesCopied += chunkRead;
        }
    }

    private void showErrorDialog(Activity activity, String task, String errorMessage) {
        Dialogs.getAlertBuilder(activity)
                .setTitle("Error " + task + ": "+ gameDownloading.title)
                .setMessage(errorMessage)
                .setCancelable(false)
                .setPositiveButton("OK", null)
                .show();
    }

    private void showDoneDialog(Activity activity, String what) {
        AlertDialog.Builder builder = Dialogs.getAlertBuilder(activity);
        builder.setTitle("Download " + what + ": "+ gameDownloading.title);
        if(what.equals("complete")) {
            builder.setPositiveButton("Start Game", (DialogInterface dialogInterface, int i) -> this.openGame(activity));
            builder.setNegativeButton("Done", null);
        } else if(what.equals("Cancelled")) {
            builder.setPositiveButton("Ok", null);
        }
        builder.show();
    }

    private void openGame(Activity activity) {
        Intent intent = new Intent("com.playstation.psm.intent.action.START_CONTENT");
        intent.putExtra("titleId", this.gameDownloading.titleId);
        intent.setData(Uri.parse("psgmpsm:play?titleId=" + this.gameDownloading.titleId));
        try {
            // wifi has to be off for psm games to run, (todo: fix this)
            WifiManager wifiManager = (WifiManager) activity.getApplicationContext().getSystemService(Context.WIFI_SERVICE);
            assert wifiManager != null;
            wifiManager.setWifiEnabled(false);

            // now start application
            activity.startActivity(intent);
        } catch (ActivityNotFoundException e) {
            showErrorDialog(activity, "Launching", "PSM app is not installed");
        }
    }
}
